'use strict';

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _yylibUi = require('yylib-ui');

var _NccPrintTplModal = require('./NccPrintTplModal');

var _NccPrintTplModal2 = _interopRequireDefault(_NccPrintTplModal);

var _ajax = require('yylib-utils/ajax');

var _ajax2 = _interopRequireDefault(_ajax);

var _jsCookie = require('js-cookie');

var _jsCookie2 = _interopRequireDefault(_jsCookie);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var field = _yylibUi.YYClass.field,
    PropTypes = _yylibUi.YYClass.PropTypes; /**
                                             * ncc打印按钮组件
                                             * @author xg
                                             */

var uiMeta = {
    name: 'ncc-printBtn',
    fields: [
    //field 参数列表：字段属性名|属性默认值|属性类型|是否必填|属性描述
    //------------打印按钮配置------------//
    field('type', 'error', PropTypes.oneOf(['primary', 'success', 'info', 'warning', 'error', '']), false, '设置按钮类型，可选值为 primary|success|info|warning|error 或者不设'), field('buttonText', '打印', PropTypes.string, false, '设置按钮的文本'), field('icon', 'file-pdf', PropTypes.string, false, '设置按钮的图标类型'), field('visible', true, PropTypes.bool, false, '设置按钮是否显示'), field('disabled', false, PropTypes.bool, false, '设置按钮是否禁用'), field('ghost', true, PropTypes.bool, false, '设置按钮为不填充形态')
    //------------打印服务配置------------//
    , field('serverUrl', window.EnvConfig.PRINT_BASE_URL, PropTypes.string, true, 'ncc后端域名'), field('queryTplUrl', '/nccloud/platform/print/queryTemplates.do', PropTypes.string, true, '查询打印模板的服务地址'), field('printUrl', '/nccloud/jzbase/jzweb/print.do', PropTypes.string, true, '执行打印的服务地址'), field('appCode', '', PropTypes.string, true, 'ncc的应用编号'), field('billId', '', PropTypes.string, true, '业务单据主键'), field('nodeKey', '', PropTypes.string, false, '打印模板的标识')]
};

var NccPrintButton = _yylibUi.YYClass.create({
    uiMeta: uiMeta,
    displayName: 'NccPrintButton',

    getInitialState: function getInitialState() {
        return {
            appCode: this.props.appCode,
            billId: this.props.billId,
            nodeKey: this.props.nodeKey,
            templates: [],
            showModal: false,
            tplId: null
        };
    },
    componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
        if ('appCode' in nextProps && nextProps.appCode !== this.props.appCode) {
            this.setState({ appCode: nextProps.appCode });
        }
        if ('billId' in nextProps && nextProps.billId !== this.props.billId) {
            this.setState({ billId: nextProps.billId });
        }
        if ('nodeKey' in nextProps && nextProps.nodeKey !== this.props.nodeKey) {
            this.setState({ nodeKey: nextProps.nodeKey });
        }
    },
    onClick: function onClick() {
        var _this = this;

        var _state = this.state,
            appCode = _state.appCode,
            billId = _state.billId,
            nodeKey = _state.nodeKey;
        var _props = this.props,
            serverUrl = _props.serverUrl,
            queryTplUrl = _props.queryTplUrl,
            printUrl = _props.printUrl;

        if (!appCode || !billId || !serverUrl || !queryTplUrl || !printUrl) {
            _yylibUi.YYMessage.error("缺少必要参数");
            return;
        }
        // 查询打印模板
        var params = {
            busiParamJson: JSON.stringify({
                funcode: appCode,
                nodekey: nodeKey,
                oids: [billId]
            }),
            sysParamJson: {
                appcode: appCode,
                pagecs: _jsCookie2.default.get("cookiets")
            }
        };
        _ajax2.default.postJSONWithCookies(serverUrl + queryTplUrl, params, function (result) {
            if (result && result.success) {
                var templates = result.data || [];
                if (templates.length === 0) {
                    _yylibUi.YYMessage.info("当前单据未配置打印模板");
                } else if (templates.length === 1) {
                    // 单个模板, 直接打印
                    _this.setState({ tplId: templates[0]["m_ctemplateid"] }, _this.onPrint);
                } else {
                    // 多个模板
                    _this.setState({ templates: templates, showModal: true });
                }
            } else {
                _yylibUi.YYMessage.error("查询打印模板失败");
            }
        }, function () {
            _yylibUi.YYMessage.error("查询打印模板失败");
        });
    },
    onTplSelected: function onTplSelected(tplId) {
        if (!tplId) {
            return;
        }
        this.setState({ showModal: false, tplId: tplId }, this.onPrint);
    },
    onCancel: function onCancel() {
        this.setState({ showModal: false });
    },
    onPrint: function onPrint() {
        var _state2 = this.state,
            appCode = _state2.appCode,
            billId = _state2.billId,
            nodeKey = _state2.nodeKey,
            tplId = _state2.tplId;
        var _props2 = this.props,
            serverUrl = _props2.serverUrl,
            printUrl = _props2.printUrl;

        var newWin = window.open('');
        if (newWin) {
            // newWin.document.title = "打印预览";
            var printForm = newWin.document.createElement('form');
            printForm.id = "print-form";
            printForm.method = "post";
            printForm.action = serverUrl + printUrl;
            printForm.style.visibility = "hidden";
            // 通过隐藏的input标签传递参数
            var funCodeInput = document.createElement('input');
            funCodeInput.type = 'hidden';
            funCodeInput.name = "funcode";
            funCodeInput.value = appCode;
            var nodeKeyInput = document.createElement('input');
            nodeKeyInput.type = 'hidden';
            nodeKeyInput.name = "nodekey";
            nodeKeyInput.value = nodeKey;
            var oidsInput = document.createElement('input');
            oidsInput.type = 'hidden';
            oidsInput.name = "oids";
            oidsInput.value = billId;
            var tempIdInput = document.createElement('input');
            tempIdInput.type = 'hidden';
            tempIdInput.name = "printTemplateID";
            tempIdInput.value = tplId;
            var appCodeInput = document.createElement('input');
            appCodeInput.type = 'hidden';
            appCodeInput.name = "sys_appcode";
            appCodeInput.value = appCode;
            printForm.append(funCodeInput);
            printForm.append(nodeKeyInput);
            printForm.append(oidsInput);
            printForm.append(tempIdInput);
            printForm.append(appCodeInput);
            newWin.document.body.appendChild(printForm);
            newWin.document.getElementById("print-form").submit();
        } else {
            _yylibUi.YYMessage.error("打印预览窗口打开失败，请检查浏览器是否阻止弹出窗口!");
        }
    },
    render: function render() {
        var _props3 = this.props,
            visible = _props3.visible,
            type = _props3.type,
            icon = _props3.icon,
            buttonText = _props3.buttonText,
            disabled = _props3.disabled,
            ghost = _props3.ghost;
        var _state3 = this.state,
            showModal = _state3.showModal,
            templates = _state3.templates;

        return _react2.default.createElement(
            'div',
            { className: this.className(), style: { display: 'inline-block' } },
            _react2.default.createElement(
                _yylibUi.YYButton,
                { icon: icon, type: type, ghost: ghost, disabled: disabled, visible: visible,
                    onClick: this.onClick },
                buttonText
            ),
            showModal === true ? _react2.default.createElement(_NccPrintTplModal2.default, { templates: templates, visible: showModal, onOk: this.onTplSelected,
                onCancel: this.onCancel }) : null
        );
    }
});
module.exports = NccPrintButton;