'use strict';

var React = require('react');

var _require = require('yylib-ui'),
    YYClass = _require.YYClass,
    YYButton = _require.YYButton,
    YYCheckbox = _require.YYCheckbox;

var PropTypes = YYClass.PropTypes;
var field = YYClass.field;

function noop() {}

var uiMeta = {
    name: 'batchbtn',
    fields: [
    //field参数列表：字段属性名|属性默认值|属性类型|是否必填|属性描述
    field('batchMode', false, PropTypes.bool, false, '初始化时是否进入批量勾选模式'), field('onBatchModeChange', null, PropTypes.function, false, '当批量勾选模式切换时的回调'), field('onCheckAllChange', null, PropTypes.function, false, '全选|反选的点击事件'), field('onExportClick', null, PropTypes.function, false, '导出按钮的点击事件'), field('onDeleteClick', null, PropTypes.function, false, '删除按钮的点击事件'), field('onSearchClick', noop, PropTypes.function, false, '删除按钮的点击事件'), field('exportBtnVisible', true, PropTypes.bool, false, '导出按钮的是否显示'), field('deleteBtnVisible', true, PropTypes.bool, false, '删除按钮的是否显示'), field('searchBtnVisible', true, PropTypes.bool, false, '高级查询的是否显示')]
};

function noop() {}

//批量编辑按钮组组件
var IjzBatchBtn = YYClass.create({
    uiMeta: uiMeta,
    getDefaultProps: function getDefaultProps() {
        return {
            batchMode: false,
            onBatchBtnClick: noop
        };
    },
    getInitialState: function getInitialState() {
        return {
            batchMode: this.props.batchMode
        };
    },
    componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
        if (this.props.batchMode != nextProps.batchMode) {
            this.setState({
                batchMode: nextProps.batchMode
            });
        }
    },
    //切换/取消批量勾选点击事件
    _changeCheckboxMode: function _changeCheckboxMode(batchMode) {
        var onBatchModeChange = this.props.onBatchModeChange;

        this.setState({
            batchMode: batchMode
        });
        if (typeof onBatchModeChange == 'function') {
            onBatchModeChange(batchMode);
        }
    },
    _rendBatchButton: function _rendBatchButton() {
        var _props = this.props,
            batchGroupList = _props.batchGroupList,
            onBatchBtnClick = _props.onBatchBtnClick;

        if (batchGroupList === null) {
            return [React.createElement(
                YYButton,
                { type: 'error', icon: 'upload', onClick: this.props.onExportClick,
                    visible: this.props.exportBtnVisible },
                '\u5BFC\u51FA'
            ), React.createElement(
                YYButton,
                { type: 'error', ghost: true, icon: 'delete', onClick: this.props.onDeleteClick,
                    visible: this.props.deleteBtnVisible },
                '\u5220\u9664'
            )];
        } else {
            var content = [];
            batchGroupList.map(function (item, index) {
                content.push(React.createElement(
                    YYButton,
                    { type: 'error', icon: item.icon, onClick: onBatchBtnClick.bind(null, item, index),
                        visible: item.visible },
                    item.name
                ));
            });
            return content;
        }
    },
    render: function render() {
        var batchMode = this.state.batchMode;
        var _props2 = this.props,
            onSearchClick = _props2.onSearchClick,
            viewMode = _props2.viewMode,
            checked = _props2.checked;

        return React.createElement(
            'div',
            { className: 'ijz-listview-buttons' },
            batchMode === false ? [React.createElement(
                YYButton,
                { className: 'batch-button', icon: 'copy', type: 'error',
                    onClick: this._changeCheckboxMode.bind(this, true) },
                '\u6279\u91CF\u64CD\u4F5C'
            ), React.createElement(
                YYButton,
                { className: 'batch-button', icon: 'search', type: 'error', visible: this.props.searchBtnVisible,
                    onClick: onSearchClick },
                '\u9AD8\u7EA7\u67E5\u8BE2'
            )] : React.createElement(
                'div',
                null,
                viewMode === 'table' ? null : React.createElement(YYCheckbox, { checked: checked, onChange: this.props.onCheckAllChange, style: { marginRight: 5 } }),
                this._rendBatchButton(),
                React.createElement(
                    YYButton,
                    { type: 'error', ghost: true, icon: 'left',
                        onClick: this._changeCheckboxMode.bind(this, false) },
                    '\u8FD4\u56DE'
                )
            )
        );
    }
});
module.exports = IjzBatchBtn;