'use strict';

var React = require('react');

var _require = require('yylib-ui'),
    YYClass = _require.YYClass,
    YYButton = _require.YYButton,
    YYTag = _require.YYTag,
    YYIcon = _require.YYIcon;

var YYFilterRow = require('../../searchfilter/YYFilterRow');
var _ = require('lodash');

var PropTypes = YYClass.PropTypes;
var field = YYClass.field;
var uiMeta = {
    name: 'ijz-filter',
    fields: [
    //field参数列表：字段属性名|属性默认值|属性类型|是否必填|属性描述
    field('filterItems', null, PropTypes.array, false, '筛选项的配置数据'), field('keepShowKeys', [], PropTypes.array, false, '一直保持显示的条件行'), field('onChange', null, PropTypes.function, false, '当前筛选条件项发生变更的回调事件'), field('minCount', 8, PropTypes.number, false, '当数据大于8条时显示更多按钮')]
};

var IjzFilter = YYClass.create({
    uiMeta: uiMeta,
    getInitialState: function getInitialState() {
        var filterItems = _.cloneDeep(this.props.filterItems);
        var currTags = this._getCurrTags(filterItems);
        return {
            currTags: currTags, //当前已选筛选条件
            filterItems: filterItems,
            isExpanded: false
        };
    },
    //获取当前已选条件项
    _getCurrTags: function _getCurrTags(filterItems) {
        var currTags = {};
        _.forEach(filterItems, function (filterItem, index) {
            if (filterItem.activekeys.length !== 0) {
                currTags[filterItem.rowKey] = _.find(filterItem.fields, ['value', filterItem.activekeys]);
            }
        });
        return currTags;
    },
    //获取当前筛选条件值
    _getCurrFilterValues: function _getCurrFilterValues() {
        var filterItems = this.state.filterItems;

        var values = {};
        _.forEach(filterItems, function (filterItem, index) {
            if (filterItem.activekeys.length !== 0) {
                values[filterItem.rowKey] = filterItem.activekeys;
            }
        });
        return values;
    },
    componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
        if ("filterItems" in nextProps) {
            var filterItems = _.cloneDeep(nextProps.filterItems);
            var currTags = this._getCurrTags(filterItems);
            this.setState({
                currTags: currTags,
                filterItems: filterItems
            });
        }
    },
    //收起|展开筛选按钮点击事件
    _onClickExpanded: function _onClickExpanded() {
        this.setState({
            isExpanded: !this.state.isExpanded
        });
    },
    //将所选择的筛选条件放在所有分类栏中显示
    _handleFilterClick: function _handleFilterClick(item, checked, rowKey) {
        var _state = this.state,
            currTags = _state.currTags,
            filterItems = _state.filterItems;

        var filterItem = _.find(filterItems, ['rowKey', rowKey]);
        if (checked) {
            //选中
            currTags[rowKey] = item;
            filterItem.activekeys = item.value;
        } else {
            //取消
            delete currTags[rowKey];
            filterItem.activekeys = '';
        }
        this.setState({ currTags: currTags, filterItems: filterItems });
        if (typeof this.props.onChange == 'function') {
            this.props.onChange(this._getCurrFilterValues());
        }
    },
    //标签关闭事件，并改变相应样式
    _handleClose: function _handleClose(tagKey, tagValue) {
        var _state2 = this.state,
            currTags = _state2.currTags,
            filterItems = _state2.filterItems;

        _.forEach(filterItems, function (filterItem, index) {
            if (filterItem.rowKey === tagKey) {
                filterItem.activekeys = '';
            }
        });
        delete currTags[tagKey];
        this.setState({ currTags: currTags, filterItems: filterItems });
        if (typeof this.props.onChange == 'function') {
            this.props.onChange(this._getCurrFilterValues());
        }
    },
    //渲染条件行
    _renderFilterViews: function _renderFilterViews() {
        var _this = this;

        var keepShowKeys = this.props.keepShowKeys;
        var filterItems = this.state.filterItems;

        var filterViews = [];
        _.forEach(filterItems, function (filterItem, index) {
            var keepShow = _.includes(keepShowKeys, filterItem.rowKey); //是否一直显示此条件行
            if (keepShow === true || filterItem.activekeys.length === 0) {
                //当前行条件未被选择则显示本条件行
                filterViews.push(React.createElement(YYFilterRow, {
                    key: "ijz_filter_row_" + index,
                    rowKey: filterItem.rowKey,
                    title: filterItem.title,
                    items: filterItem.fields,
                    multiple: false,
                    defaultActivedKey: filterItem.activekeys,
                    onClick: _this._handleFilterClick,
                    minCount: _this.props.minCount
                }));
            }
        });
        return filterViews.length ? filterViews : null;
    },
    //渲染当前已选项
    _renderCurrTags: function _renderCurrTags() {
        var _this2 = this;

        var currTags = this.state.currTags;

        var tagViews = [];
        _.forEach(currTags || {}, function (tagItem, tagKey) {
            return tagViews.push(React.createElement(
                YYTag,
                { key: "ijz_filter_tag_" + tagKey, className: tagItem.disabled ? 'not-close' : '', closable: tagItem.disabled ? false : true,
                    afterClose: function afterClose() {
                        return _this2._handleClose(tagKey, tagItem.value);
                    } },
                tagItem.text
            ));
        });
        return tagViews.length ? tagViews : null;
    },
    render: function render() {
        var isExpanded = this.state.isExpanded;

        return React.createElement(
            'div',
            { className: this.className() },
            React.createElement(
                'div',
                { className: 'ijz-filter-selected' + (isExpanded ? '' : ' hidden') },
                _.isEmpty(this.state.currTags) ? React.createElement('span', { dangerouslySetInnerHTML: { __html: '&nbsp;' } }) : React.createElement(
                    'span',
                    null,
                    '\u5F53\u524D\u7B5B\u9009\uFF1A'
                ),
                this._renderCurrTags(),
                React.createElement(
                    YYButton,
                    { type: 'error', ghost: true, className: 'ijz-filter-showbtn', onClick: this._onClickExpanded },
                    isExpanded ? '收起筛选' : '展开筛选',
                    React.createElement(YYIcon, { type: isExpanded ? 'up' : 'down' })
                )
            ),
            isExpanded ? this._renderFilterViews() : null
        );
    }
});
module.exports = IjzFilter;