'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var React = require('react');
var _ = require('lodash');

var _require = require('yylib-ui'),
    YYClass = _require.YYClass,
    YYForm = _require.YYForm,
    YYRow = _require.YYRow,
    YYCol = _require.YYCol;

var FormItemType = require('./FormItemType');
var formatUtils = require('../../utils/formatUtils');
var MathX = require('../../utils/matheval/formula_evaluator');
var MathUtil = require('../../utils/MathUtil');
//匹配出计算公式中的字段
function getFactorFromFormula(mathFormula) {
    var r = new RegExp("\\{(.+?)\\}", "igm");
    return mathFormula.match(r);
}
// 运行计算公式
function mathFormulaResult(mathFormula, rowData, defaultValue) {
    var formula = '' + mathFormula;
    _.forIn(rowData, function (value, key) {
        formula = formula.replace(new RegExp('{' + escape(key) + '}', 'gm'), value ? escape(value) : 0);
    });
    try {
        var result = MathX.eval(formula);
        // 判断计算结果是否为数值类型
        if (new RegExp("(^[0-9]+\.?[0-9]*$)|(^\-[0-9]+\.?[0-9]*$)").test(result)) {
            result = _.toNumber(result);
        } else {
            result = 0;
        }
        return result;
    } catch (error) {
        console.error('[公式计算出错]', { formula: formula, mathFormula: mathFormula, rowData: rowData, error: error });
    }
    return defaultValue;
}
// 处理参照映射项赋值
function dealReferRelation(options) {
    var _ref = options || {},
        form = _ref.form,
        childProps = _ref.childProps,
        key = _ref.key,
        fullValue = _ref.fullValue,
        referRelation = _ref.referRelation;

    var _ref2 = form || {},
        getFieldsValue = _ref2.getFieldsValue,
        setFieldsValue = _ref2.setFieldsValue;

    if (!getFieldsValue || !setFieldsValue || !referRelation || !fullValue) {
        return;
    }
    console.log("[" + key + "]执行参照映射项赋值");
    var fieldsValue = getFieldsValue();
    for (var prop in referRelation) {
        if (referRelation) var relation = referRelation[prop];
        var props = childProps[prop];
        fieldsValue[prop] = parseData({ props: props, relation: relation, fullValue: fullValue });
    }
    setFieldsValue(fieldsValue);
}

// 检查值与类型是否匹配,数据转换
function parseData(options) {
    var _ref3 = options || {},
        props = _ref3.props,
        fullValue = _ref3.fullValue,
        relation = _ref3.relation;

    if (!props || !fullValue || !relation) {
        return;
    }
    if (FormItemType.inputrefer === relation.type) {
        //  参照
        // 配置后修改设计器字段类型
        if (relation.type !== props.uisubtype) {
            console.error("[" + props.uikey + "]字段类型不匹配,请检查配置");
            return null;
        }
        var data = {};
        data.id = fullValue ? fullValue['' + relation.id] : null;
        data.code = fullValue ? fullValue['' + relation.code] : null;
        data.name = fullValue ? fullValue['' + relation.name] : null;
        if (!data.id || !data.code || !data.name) {
            console.error("为[" + props.uikey + "]字段赋值[" + data + "]失败: 请检查配置");
            return null;
        }
        return props.multiselect == true ? [data] : data;
    } else if (FormItemType.enumselect === relation.type) {
        // 档案枚举
        // 配置后修改设计器字段类型
        if (relation.type !== props.uisubtype) {
            console.error("[" + props.uikey + "]字段类型不匹配,请检查配置");
            return null;
        }
        var data = {};
        data.id = fullValue ? fullValue['' + relation.id] : null;
        data.code = fullValue ? fullValue['' + relation.code] : null;
        data.name = fullValue ? fullValue['' + relation.name] : null;
        if (!data.id || !data.code || !data.name) {
            console.error("为[" + props.uikey + "]字段赋值[" + data + "]失败: 请检查配置");
            return null;
        }
        if (props.valueType === 'object') {
            // 取值类型为对象
            if (props.valueKey !== "id") {
                delete data.id;
            }
        } else {
            // 取值类型为字符串
            if (props.valueKey === "id") {
                data = data.id;
            } else {
                data = data.code;
            }
        }
        return props.multiselect == true ? [data] : data;
    } else {
        var value = fullValue['' + relation.code];
        if (!value) {
            return value;
        }
        try {
            switch (props.uisubtype) {
                case FormItemType.number:
                    // 整数框
                    if (MathUtil.isInteger(value)) {
                        value = parseInt(value);
                    } else {
                        console.error("[" + props.uikey + "]字段赋值[" + fullValue['' + relation.code] + "]失败:数据类型不匹配");
                        value = null;
                    }
                    break;
                case FormItemType.currency:
                    // 精确数值框
                    if (new RegExp("^[0-9]+\.?[0-9]*$").test(value)) {
                        value = _.toNumber(value);
                    } else {
                        console.error("[" + props.uikey + "]字段赋值[" + fullValue['' + relation.code] + "]失败:数据类型不匹配");
                        value = null;
                    }
                    break;
                case FormItemType.date:
                    // 日期框
                    value = formatUtils.parseAntDateWithFormat(value, props.format);
                    if (!value || !value.getYear()) {
                        console.error("[" + props.uikey + "]字段赋值[" + fullValue['' + relation.code] + "]失败:数据类型不匹配");
                        value = null;
                    }
                    break;
                // TODO 后续有实际需求时完善
                case FormItemType.time: // 时间框
                case FormItemType.radiogroup: // 单选框
                case FormItemType.checkboxgroup: // 复选框
                case FormItemType.cascader: // 下拉级联
                case FormItemType.select: // 下拉选择
                case FormItemType.autoselect:
                    // 自动匹配
                    console.warn("[" + props.uikey + "]字段赋值失败:暂不支持该类型组件");
                    value = null;
                    break;
                default:
                    break;
            }
        } catch (e) {
            console.error("[" + props.uikey + "]字段赋值[" + value + "]失败:数据转换失败," + e.message);
            value = null;
        }
        return value;
    }
}

var FormWidget = YYClass.create({
    _getCloneProps: function _getCloneProps(target) {
        var props = {
            form: this.props.form,
            labelCol: this.props.labelCol,
            wrapperCol: this.props.wrapperCol
        };

        if (target.props.disabled != undefined) {
            props.disabled = target.props.disabled;
        } else {
            props.disabled = this.props.disabled;
        }

        if (target.props.labelCol) {
            props.labelCol = target.props.labelCol;
        }

        if (target.props.wrapperCol) {
            props.wrapperCol = target.props.wrapperCol;
        }

        return props;
    },
    _renderChildren: function _renderChildren(children, colnumber) {
        var _props$form = this.props.form,
            getFieldProps = _props$form.getFieldProps,
            setFieldsValue = _props$form.setFieldsValue,
            getFieldsValue = _props$form.getFieldsValue;

        var _this = this;
        var colSpan = Math.floor(24 / colnumber);
        var colViews = [];
        //子集不存在的相关属性值则继承父form的相关属性值
        var doCloneElement = function doCloneElement(target) {
            var props = _this._getCloneProps(target);
            return React.cloneElement(target, props);
        };
        //记录下配置的计算公式-------add by wuhao----------
        var formulas = [];
        var childProps = {};
        _.forEach(children, function (child) {
            if (child.props.mathFormula) {
                _.forEach(getFactorFromFormula(child.props.mathFormula), function (fieldName) {
                    // formulas[fieldName.substring(1, fieldName.length - 1)] = formulaInfo
                    formulas.push({
                        key: fieldName.substring(1, fieldName.length - 1),
                        formulaInfo: child.props.mathFormula,
                        resultkey: child.props.uikey
                    });
                });
            }
            childProps['' + child.props.uikey] = child.props;
        });
        //-------add by wuhao----------end
        //记录当前总span值
        var totalSpan = 0;
        _.forEach(children, function (child, rowIndex) {
            if (!React.isValidElement(child)) {
                console.warn('FormWidget无法解析非React对象元素', child);
            } else {
                //--------add by wuhao----------
                var resultKeySet = new Set();
                var currentFieldFormulas = _.filter(formulas, function (formulaField) {
                    resultKeySet.add(formulaField.resultkey);
                    //返回计算公式
                    return formulaField.key == child.props.uikey;
                });
                if (currentFieldFormulas && currentFieldFormulas.length) {
                    var _onChange = null;
                    if (child.props.onChange) {
                        _onChange = child.props.onChange;
                    }
                    //防止重复绑定onChange导致多次执行的问题
                    if (!child.props.onChangeWithMath) {
                        child.props.onChange = function (value) {
                            var mathResult = null;
                            var fieldsValue = getFieldsValue();
                            var fieldsValurOrigin = _.cloneDeep(fieldsValue); //计算前的表单值
                            _.forEach(currentFieldFormulas, function (formulas) {
                                //跟公式中的某一个计算列匹配时进行计算
                                fieldsValue[formulas.key] = value;
                                mathResult = mathFormulaResult(formulas.formulaInfo, fieldsValue, 0);
                                //数字类型默认格式化显示千分位,注意cellText=0的情况
                                if (typeof mathResult === 'number' || mathResult) {
                                    if (child.props.fieldtype == 'currency') {
                                        //精确数值
                                        var numStrs = (mathResult + "").split(".");
                                        var decimal = numStrs.length == 2 ? numStrs[1].length : 0; //保留原始小数位
                                        if (child.props.decimal >= 0) decimal = child.props.decimal; //保留列定义的小数位
                                        mathResult = formatUtils.formatNumber(mathResult, decimal, child.props.comma === false ? 0 : 3); //格式化千分位
                                    }
                                }
                                fieldsValue[formulas.resultkey] = mathResult;
                                setFieldsValue(fieldsValue);
                            });
                            if (_.isFunction(_onChange)) {
                                _onChange(value);
                            }
                            //执行结果字段上绑定的 onMathFormula 方法
                            var resultKeyChildren = _.filter(children, function (child) {
                                return Array.from(resultKeySet).includes(child.props.uikey);
                            });
                            _.forEach(resultKeyChildren, function (resultKeyChild) {
                                if (_.isFunction(resultKeyChild.props.onMathFormula)) {
                                    resultKeyChild.props.onMathFormula(fieldsValurOrigin[resultKeyChild.props.uikey], mathResult);
                                }
                            });
                        };
                        //将onChange保存下
                        child.props.onChangeWithMath = true;
                    }
                }
                //--------add by wuhao----------end
                // add begin by xg 处理参照映射项赋值
                if (FormItemType.inputrefer === child.props.uisubtype && _.isObject(child.props.referRelation) && !_.isEmpty(child.props.referRelation)) {
                    if (child.props.multiselect == true) {
                        console.error("[" + child.props.uikey + "]字段参照映射失败: 参照多选时无法进行赋值操作");
                    } else {
                        var _onChange = child.props.onChange;
                        //防止重复绑定onChange导致多次执行的问题
                        if (!child.props.onChangeWithRelation) {
                            child.props.onChange = function (shortValue, keyField, fullValue, selectedNode) {
                                // 1.处理参照映射项赋值
                                dealReferRelation({ form: _this.props.form, childProps: childProps, key: child.props.uikey, fullValue: fullValue, referRelation: child.props.referRelation });
                                // 2.执行自定义的onChange事件
                                if (_.isFunction(_onChange)) {
                                    _onChange(shortValue, keyField, fullValue, selectedNode);
                                }
                            };
                            child.props.onChangeWithRelation = true;
                        }
                    }
                }
                // add end by xg
                if (child.props.uitype == 'FormItemWidget' || child.props.uitype == 'MDFormItem') {
                    //FormItemWidget
                    var span = colSpan; //默认布局一行N列
                    if (child.props.rowBreak || child.props.newline) {
                        //强制换行或独占一行时，需要补全上一行的空缺单元
                        var spaceSpan = 24 - totalSpan % 24; //计算补全当前行需要填充的span值
                        if (spaceSpan > 0 && spaceSpan < 24) {
                            //填充一个占位单元补全整行
                            colViews.push(React.createElement(YYCol, { span: spaceSpan, className: 'space', style: { height: 45 } }));
                            totalSpan = totalSpan + spaceSpan;
                        }
                    }
                    if (child.props.newline) {
                        //独立一行
                        span = 24;
                    }
                    var colStyle = {};
                    if (child.props.visible == false) {
                        colStyle.display = "none"; //是否隐藏本单元
                    } else {
                        totalSpan = totalSpan + span; //注意：不显示的单元格不参与计算
                    }
                    colViews.push(React.createElement(
                        YYCol,
                        { key: "fromwidget_col_" + child.props.nid, span: span, className: child.props.className, style: colStyle },
                        doCloneElement(child)
                    ));
                } else {
                    //其它自定义组件，注意把form提供给自定义组件
                    colViews.push(React.cloneElement(child, { form: _this.props.form }));
                }
            }
        });
        return React.createElement(
            YYRow,
            { className: 'yyui-formwidget-rows' },
            colViews
        );
    },

    render: function render() {
        var _props = this.props,
            children = _props.children,
            colnumber = _props.colnumber;

        return React.createElement(
            YYForm,
            _extends({}, this.props, { form: null }),
            this._renderChildren(children, colnumber)
        );
    }
});

FormWidget = YYForm.create()(FormWidget);
module.exports = FormWidget;