'use strict';

var React = require('react');

var _require = require('react-dom'),
    findDOMNode = _require.findDOMNode;

var Trigger = require('rc-trigger');
var YYClass = require('../base/YYClass');
var YYInput = require('../formcontrols/input/YYInput');
var YYIcon = require('../icon/YYIcon');
var YYCalc = require('./YYCalc');
var CSSUtil = require('../../utils/CSSUtil');
var PropTypes = YYClass.PropTypes;
var field = YYClass.field;
require('./style/index2.less');
//定义META
var uiMeta = {
    name: 'calcinput',
    fields: [field('size', 'default', PropTypes.string, false, '设置input输入框大小，给定三种尺寸,small,default,large')]
};
var builtinPlacements = {
    bottom: {
        points: ['tc', 'bc']
    }
};
var YYCalcInput = YYClass.create({
    uiMeta: uiMeta,
    getInitialState: function getInitialState() {
        return {
            value: '',
            preValue: '',
            placement: 'bottom'
        };
    },
    changeValue: function changeValue(event) {
        var _this = this;

        var value = event.target.value;
        if (isNaN(value)) {
            if (value === '.' || value == '0..') value = '0.';else value = this.state.preValue;
        } else {
            if (value === '0') this.state.preValue = '0';else this.state.preValue = value;
        }
        if (value.indexOf('.') < 0) value = value.replace(/^[0]+/, "");
        this.setState({
            value: value
        }, function () {
            _this.refs.yycalc.setState({
                input_num: value,
                opr: '',
                formula: '',
                result: ''
            });
        });
        if (value === undefined || value === '') {
            var inputDom = findDOMNode(this);
            inputDom.querySelector('.yyui-input + .yyui-icon').style.display = 'inline';
        } else {
            var _inputDom = findDOMNode(this);
            _inputDom.querySelector('.yyui-input + .yyui-icon').style.display = 'none';
        }
    },
    onEqual: function onEqual(result) {
        this.setState({
            value: result
        });
    },
    onCleanInput: function onCleanInput() {
        var _this2 = this;

        this.setState({
            value: ''
        }, function () {
            _this2.refs.yycalc.clearState();
        });
    },
    onMouseOverInput: function onMouseOverInput(e) {
        if (e.target.value !== '') {
            var inputDom = findDOMNode(this);
            inputDom.querySelector('.yyui-input + .yyui-icon').style.display = 'none';
        }
    },
    onMouseOutInput: function onMouseOutInput(e) {
        var inputDom = findDOMNode(this);
        inputDom.querySelector('.yyui-input + .yyui-icon').style.display = 'inline';
    },
    _getPopupContainer: function _getPopupContainer(e) {
        if (typeof this.props.getPopupContainer == 'function') {
            return this.props.getPopupContainer(e);
        } else {
            var parentPage = CSSUtil.parent(e, ".yyui-page");
            if (parentPage) {
                return parentPage;
            } else {
                return document.body;
            }
        }
    },
    render: function render() {
        return React.createElement(
            'div',
            { className: this.className(), ref: 'YYCalcInput' },
            React.createElement(
                Trigger,
                { popup: React.createElement(YYCalc, { onEqual: this.onEqual, ref: 'yycalc' }),
                    action: ['click'],
                    getPopupContainer: this._getPopupContainer,
                    popupPlacement: this.state.placement,
                    builtinPlacements: builtinPlacements
                },
                React.createElement(
                    'div',
                    { className: 'calcinput-area', onMouseEnter: this.onMouseOverInput,
                        onMouseLeave: this.onMouseOutInput },
                    React.createElement(YYInput, { onChange: this.changeValue,
                        value: this.state.value,
                        size: this.props.size,
                        onClearClick: this.onCleanInput
                    }),
                    React.createElement(YYIcon, { type: 'calculator' })
                )
            )
        );
    }
});
module.exports = YYCalcInput;