'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var React = require('react');
var YYClass = require('../../base/YYClass');
var PropTypes = YYClass.PropTypes;
var field = YYClass.field;

var _require = require('antd'),
    Cascader = _require.Cascader;

var _ = require('lodash');
var ajax = require('../../../utils/ajax');
var DataUtil = require('../../../utils/DataUtil');
require('./style/index.less');
//定义META
var uiMeta = {
    name: 'cascader',
    fields: [
    // field参数列表：字段属性名|属性默认值|属性类型|是否必填|属性描述
    field('options', null, PropTypes.object, false, '本地数据项'), field('dataurl', null, PropTypes.string, false, '请求远程数据源的URL，注意当配置了dataurl时，options属性将失效'), field('dataparams', null, PropTypes.object, false, '请求远程数据时的自定义参数'), field('defaultValue', undefined, PropTypes.array, false, '默认的选中项'), field('value', undefined, PropTypes.array, false, '指定选中项'), field('style', {}, PropTypes.object, false, '自定义样式'), field('className', '', PropTypes.string, false, '自定义类名'), field('popupClassName', '', PropTypes.string, false, '自定义浮层类名'), field('placeholder', '请选择', PropTypes.string, false, '输入框占位文本'), field('size', 'default', PropTypes.oneOf(['large', 'default', 'small']), false, '输入框大小'), field('disabled', false, PropTypes.bool, false, '禁用'), field('allowClear', true, PropTypes.bool, false, '是否支持清除'), field('changeOnSelect', false, PropTypes.bool, false, '当此项为 true 时，点选每级菜单选项值都会发生变化，具体见上面的演示'), field('popupPlacement', 'bottomLeft', PropTypes.oneOf(['bottomLeft', 'bottomRight', 'topLeft', 'topRight']), false, '浮层预设位置'), field('expandTrigger', 'click', PropTypes.oneOf(['click', 'hover']), false, '次级菜单的展开方式'), field('onChange', null, PropTypes.func, false, '选择完成后的回调'), field('displayRender', null, PropTypes.func, false, '选择后展示的渲染函数'), field('onChangeValueType', 'string', PropTypes.string, false, 'onChange或赋值时的取值类型，与options中的数据结构无关，options类型是数组对象; 在表单中默认是string，编辑表格中默认是object'), field('isRefreshCache', false, PropTypes.bool, false, '是否刷新缓存,默认false不刷新'), field('rootcode', null, PropTypes.string, false, '标识从哪个层级开始')]
};

var YYCascader = YYClass.create({
    uiMeta: uiMeta,
    getInitialState: function getInitialState() {
        var _props = this.props,
            value = _props.value,
            defaultValue = _props.defaultValue;

        return {
            dataurl: this.props.dataurl,
            dataparams: this.props.dataparams,
            rootcode: this.props.rootcode,
            value: value ? value : defaultValue,
            options: this.props.options
        };
    },
    componentDidMount: function componentDidMount() {
        var _props2 = this.props,
            options = _props2.options,
            isRefreshCache = _props2.isRefreshCache,
            nid = _props2.nid;

        if (!options || _.isEmpty(options)) {
            if (isRefreshCache == true) {
                this._requestData();
            } else {
                var cacheData = DataUtil.localRead('YYCascader_' + nid);
                if (cacheData) {
                    this.setState({ options: JSON.parse(cacheData) });
                } else {
                    this._requestData();
                }
            }
        }
    },
    componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
        var _props3 = this.props,
            dataurl = _props3.dataurl,
            dataparams = _props3.dataparams,
            rootcode = _props3.rootcode,
            value = _props3.value,
            options = _props3.options;
        // value优先级高于defaultValue

        if (nextProps.defaultValue && !_.isEqual(nextProps.defaultValue, value)) {
            this.setState({ value: nextProps.defaultValue }, this._dealValueWithRootCode);
        }
        if (nextProps.value && !_.isEqual(nextProps.value, value)) {
            this.setState({ value: nextProps.value }, this._dealValueWithRootCode);
        }
        if (nextProps.rootcode && !_.isEqual(nextProps.rootcode, rootcode)) {
            this.setState({ rootcode: nextProps.rootcode }, this._dealValueWithRootCode);
        }
        // options属性优先级高于根据dataurl查询
        if (nextProps.options && !_.isEqual(nextProps.options, options)) {
            this.setState({ options: nextProps.options });
        } else {
            if (nextProps.dataurl && !_.isEqual(nextProps.dataurl, dataurl) || nextProps.dataparams && !_.isEqual(nextProps.dataparams, dataparams)) {
                this.setState({ dataurl: nextProps.dataurl, dataparams: nextProps.dataparams }, this._requestData);
            }
        }
    },
    _dealValueWithRootCode: function _dealValueWithRootCode() {
        var _state = this.state,
            value = _state.value,
            rootcode = _state.rootcode;

        if (value && value.length && rootcode) {
            var newValue = _.cloneDeep(value);
            var index = newValue.indexOf(rootcode);
            if (index !== -1) {
                newValue = _.slice(newValue, index, newValue.length);
            }
            this.setState({ value: newValue });
        }
    },
    _requestData: function _requestData() {
        var _state2 = this.state,
            dataurl = _state2.dataurl,
            dataparams = _state2.dataparams,
            rootcode = _state2.rootcode;

        var _this = this;
        if (dataurl) {
            if (dataparams) {
                if (rootcode) {
                    dataparams = _.assign({}, dataparams, { rootcode: rootcode });
                }
                ajax.getJSON(dataurl, dataparams, function (result) {
                    _this.setState({ options: result }, function () {
                        DataUtil.localSave('YYCascader_' + _this.props.nid, result);
                    });
                });
            } else {
                ajax.getJSON(dataurl, function (result) {
                    _this.setState({ options: result }, function () {
                        DataUtil.localSave('YYCascader_' + _this.props.nid, result);
                    });
                });
            }
        }
    },
    handleKeyDown: function handleKeyDown(e) {
        var code = e.keyCode;
        var cascader = this.refs.cascader;
        if (code == 13) {
            cascader.setState({
                popupVisible: true
            });
        }
    },
    _treeToArr: function _treeToArr(options) {
        var _this2 = this;

        var optionArr = [];
        _.forEach(options, function (option) {
            if (_.isArray(option.children)) {
                optionArr.push(option);
                optionArr = optionArr.concat(_this2._treeToArr(option.children));
            } else {
                delete option.children;
                optionArr.push(option);
            }
        });
        return optionArr;
    },
    _onChange: function _onChange(value, selectOptions) {
        var _this3 = this;

        var onChangeValueType = this.props.onChangeValueType;

        var changeValue = [];
        var options = this.props.dataurl ? this.state.options : this.props.options;
        var optionArr = this._treeToArr(options);
        // var optionArr = this._treeToArr(_.cloneDeep(options))
        if (_.isArray(value)) {
            _.forEach(value, function (valueItem) {
                if (onChangeValueType == 'object') {
                    changeValue.push(_this3._findValueByPropKey(optionArr, valueItem));
                } else {
                    changeValue.push(valueItem);
                }
            });
        } else {
            changeValue = value;
        }
        this.setState({ value: changeValue });
        if (_.isFunction(this.props.onChange)) {
            this.props.onChange(changeValue, selectOptions);
        }
    },
    _findValueByPropKey: function _findValueByPropKey(options, value) {
        var option = _.find(options, { 'value': value });
        // var { valueKey, textKey } = this.props
        // var option = _.find(options, [valueKey, value]);
        // var valueObj = {};
        // if(option){
        //     valueObj[valueKey] = option[valueKey];//键值
        //     valueObj[textKey] = option[textKey]//文本值
        // }
        return option;
    },
    render: function render() {
        var options = this.props.dataurl ? this.state.options : this.props.options;
        return React.createElement(
            'div',
            { className: this.className(), onKeyDown: this.handleKeyDown },
            React.createElement(
                Cascader,
                _extends({}, this.props, { options: options, value: this.state.value, ref: 'cascader', onChange: this._onChange }),
                this.props.children
            )
        );
    }
});

module.exports = YYCascader;