'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var React = require('react');
var YYClass = require('../base/YYClass');
var YYButton = require('../button/YYButton');
var YYIcon = require('../icon/YYIcon');
var ajax = require('../../utils/ajax');
var _ = require('lodash');

var Trigger = require('rc-trigger');
var YYInputButton = require('../formcontrols/inputbutton/YYInputButton');

function PopToolbar(props) {
    return props.searchMultiple ? React.createElement(
        'div',
        { className: 'popupcontent-toolbar' },
        React.createElement(
            YYButton,
            { size: 'small', onClick: props.onOk },
            '\u786E\u5B9A'
        )
    ) : null;
}

function PopContent(props) {
    var _this2 = this;

    var popContent = [];
    var nodes = props.nodes,
        curHover = props.curHover,
        seletedContentKeys = props.seletedContentKeys,
        searchMultiple = props.searchMultiple,
        onChooseContent = props.onChooseContent,
        onHoverContent = props.onHoverContent;

    _.forEach(nodes, function (node, index) {
        var className = curHover === index + 1 ? 'popup-content hover' : 'popup-content';
        var id = node.id,
            name = node.name; //节点格式要求

        if (searchMultiple) {
            //标出已勾选的
            var isChecked = _.includes(seletedContentKeys, id);
            popContent.push(React.createElement(
                'li',
                { name: name, className: className, onClick: onChooseContent.bind(_this2, node), 'data-index': index + 1, onMouseOver: onHoverContent },
                React.createElement(
                    'label',
                    { htmlFor: id },
                    React.createElement('input', { type: 'checkbox', id: id, checked: isChecked }),
                    React.createElement(
                        'span',
                        { onClick: onChooseContent.bind(_this2, node) },
                        name
                    )
                )
            ));
        } else {
            popContent.push(React.createElement(
                'li',
                { 'data-index': index + 1,
                    id: id,
                    name: name,
                    onMouseOver: onHoverContent,
                    onClick: onChooseContent.bind(_this2, node),
                    className: className },
                name
            ));
        }
    });
    return React.createElement(
        'ul',
        { className: 'popupcontent-wrapper' },
        popContent
    );
}
var uiMeta = {
    name: 'tree-searchbar',
    fields: []
};
var YYTreeSearchTrigger = YYClass.create({
    uiMeta: uiMeta,
    getInitialState: function getInitialState() {
        return {
            triggerVisible: false,
            curHover: null,
            seletedContentKeys: [],
            seletedContents: [],
            filterNodes: []
        };
    },
    // 回车事件
    _onPressEnter: function _onPressEnter(e) {
        if (e && e.target && e.target.value) {
            this._doQuery(e.target.value);
        }
    },
    // 搜索按钮点击事件
    _onButtonClick: function _onButtonClick(value) {
        if (value) {
            this._doQuery(value);
        }
    },
    // 查询数据
    _doQuery: function _doQuery(searchText) {
        var _ref = this.props || {},
            searchUrl = _ref.searchUrl,
            maxSize = _ref.maxSize,
            onSearchParams = _ref.onSearchParams,
            onSearchSuccess = _ref.onSearchSuccess,
            onSearchFail = _ref.onSearchFail,
            condition = _ref.condition;

        if (!searchUrl) {
            return;
        }
        var params = { searchText: searchText, limit: maxSize };
        if (_.isFunction(onSearchParams)) {
            var defParams = onSearchParams(_.cloneDeep(params));
            if (defParams && !_.isEmpty(defParams)) {
                // 避免onSearchParams回调函数没有返回值
                params = defParams;
            }
        }
        var _condition = ajax.getParams(condition);
        if (!_.isEmpty(_condition) && _.isPlainObject(_condition)) {
            //转成JSON格式字符串
            params.condition = JSON.stringify(_condition);
        }
        var _this = this;
        ajax.getJSON(searchUrl, params, function (result) {
            _this.setState({ filterNodes: result, triggerVisible: true }, function () {
                if (_.isFunction(onSearchSuccess)) {
                    onSearchSuccess(result);
                }
            });
        }, function (result) {
            var errorMsg = result && result.error ? result.error.message : null;
            _this.setState({ filterNodes: [], triggerVisible: false }, function () {
                if (_.isFunction(onSearchFail)) {
                    onSearchFail(errorMsg);
                }
            });
        });
    },
    // 清除
    _onClearClick: function _onClearClick() {
        this.setState({ filterNodes: [], triggerVisible: false, seletedContentKeys: [], seletedContents: [] });
    },

    _onHoverContent: function _onHoverContent(e) {
        var curHover = e.target && e.target.dataset.index ? e.target.dataset.index : this.state.curHover;
        this.setState({ curHover: Number(curHover) });
    },
    _onChooseContent: function _onChooseContent(treeNode, e) {
        var _this3 = this;

        if (this.props.checkable) {
            var _state = this.state,
                seletedContentKeys = _state.seletedContentKeys,
                seletedContents = _state.seletedContents;

            if (_.includes(seletedContentKeys, treeNode.id)) {
                _.remove(seletedContentKeys, function (id) {
                    return treeNode.id === id;
                });
                _.remove(seletedContents, function (selectedContent) {
                    return selectedContent.id === treeNode.id;
                });
            } else {
                seletedContentKeys.push(treeNode.id);
                seletedContents.push(treeNode);
            }
            this.setState({ seletedContents: seletedContents, seletedContentKeys: seletedContentKeys });
        } else {
            this.setState({ triggerVisible: false }, function () {
                if (_.isFunction(_this3.props.onSearchChoose)) {
                    _this3.props.onSearchChoose(treeNode.id, treeNode);
                }
            });
        }
    },
    _onOk: function _onOk() {
        var _this4 = this;

        this.setState({ triggerVisible: false }, function () {
            if (_.isFunction(_this4.props.onSearchChoose)) {
                _this4.props.onSearchChoose(_this4.state.seletedContentKeys, _this4.state.seletedContents);
            }
        });
    },
    render: function render() {
        var _this5 = this;

        var _state2 = this.state,
            triggerVisible = _state2.triggerVisible,
            filterNodes = _state2.filterNodes,
            curHover = _state2.curHover,
            seletedContentKeys = _state2.seletedContentKeys;
        var checkable = this.props.checkable;

        var popupView = filterNodes && !_.isEmpty(filterNodes) ? React.createElement(
            'div',
            null,
            React.createElement(PopContent, { nodes: filterNodes,
                curHover: curHover,
                seletedContentKeys: seletedContentKeys,
                searchMultiple: checkable,
                onChooseContent: this._onChooseContent,
                onHoverContent: this._onHoverContent
            }),
            React.createElement(PopToolbar, { onOk: this._onOk,
                searchMultiple: this.props.checkable
            })
        ) : React.createElement(
            'div',
            { style: { width: 150, textAlign: 'center', padding: '40px 0' } },
            React.createElement(YYIcon, { type: 'frown' }),
            React.createElement(
                'span',
                { style: { marginLeft: 3, color: '#999' } },
                '\u6682\u65E0\u6570\u636E'
            )
        );
        return React.createElement(
            Trigger,
            { maskClosable: true,
                action: ["focus"],
                popupAlign: { points: ['tl', 'bl'] },
                popupClassName: 'yyui-tree-search-trigger',
                onPopupVisibleChange: this._onPopupVisibleChange,
                popupVisible: triggerVisible,
                popup: popupView
            },
            React.createElement(YYInputButton, _extends({}, this.props.inputProps, {
                onFocus: function onFocus(e, value) {
                    e.stopPropagation();
                    var triggerVisible = filterNodes && filterNodes.length;
                    _this5.setState({ triggerVisible: triggerVisible });
                },
                className: this.className(),
                onChange: this._onClearClick,
                onPressEnter: this._onPressEnter,
                buttonClick: this._onButtonClick,
                onClearClick: this._onClearClick,
                placeholder: this.props.searchPlaceholder,
                buttonText: '\u641C\u7D22', buttonIcon: null }))
        );
    }
});

module.exports = YYTreeSearchTrigger;